use std::{fs::File, path::PathBuf};
use slint::{Rgba8Pixel, SharedPixelBuffer};
use tool_helper::Error;

pub struct TIMInfo {
    image_data: SharedPixelBuffer<Rgba8Pixel>,
    palette:    Option<Vec<Rgba8Pixel>>,
}

impl TIMInfo {
    pub fn from_image(path: &PathBuf) -> Result<TIMInfo, Error> {
        fn make_color(iter: &mut dyn std::iter::Iterator<Item = &u8>) -> Option<Rgba8Pixel> { 
            Some(Rgba8Pixel::new(
                *iter.next()?, *iter.next()?, *iter.next()?, 
                0xFF))
        }
    
        let mut reader = png::Decoder::new(File::open(path)?).read_info().or_else(|error| {Err(Error::from_error(error))})?;
        let     info   = reader.info().clone();
        
        let mut buffer     = vec![0; reader.output_buffer_size()];
        let     frame_info = reader.next_frame(&mut buffer).or_else(|error| {Err(Error::from_error(error))})?;
        let     bit_depth  = frame_info.bit_depth;
        let mut image_data = SharedPixelBuffer::new(frame_info.width, frame_info.height);
    
        if info.color_type == png::ColorType::Indexed {
            let palette = info.palette.ok_or(Error::from_str("Found indexed PNG without palette"))?;
            let mut palette_colors = Vec::new();
            let mut iter           = palette.iter();
    
            while let Some(color) = make_color(&mut iter) {
                palette_colors.push(color);
            }
    
            let mut dst_pixels  = image_data.make_mut_slice();
            for byte in buffer.into_iter() {
                match bit_depth {
                    png::BitDepth::Four  => {
                        dst_pixels[0] = palette_colors[(byte >>  4) as usize];
                        dst_pixels[1] = palette_colors[(byte & 0xF) as usize];
                        dst_pixels    = &mut dst_pixels[2..];
                    },
                    png::BitDepth::Eight => {
                        dst_pixels[0] = palette_colors[byte as usize];
                        dst_pixels    = &mut dst_pixels[1..];
                    },
                    _                    => {return Err(Error::from_str("Only 4 and 8bit color depth are supported"));}
                }
            }

            Ok(TIMInfo{image_data, palette: Some(palette_colors)})
        }
    
        else {
            Err(Error::not_implemented("Support for non indexed images"))
        }
    }

    pub fn get_slint_images(&self) -> slint::Image {
        slint::Image::from_rgba8_premultiplied(self.image_data.clone())
    }
}